/*****************************************************************************\
|                                               ( )_  _                       |
|    _ _    _ __   _ _    __    ___ ___     _ _ | ,_)(_)  ___   ___     _     |
|   ( '_`\ ( '__)/'_` ) /'_ `\/' _ ` _ `\ /'_` )| |  | |/',__)/' v `\ /'_`\   |
|   | (_) )| |  ( (_| |( (_) || ( ) ( ) |( (_| || |_ | |\__, \| (˅) |( (_) )  |
|   | ,__/'(_)  `\__,_)`\__  |(_) (_) (_)`\__,_)`\__)(_)(____/(_) (_)`\___/'  |
|   | |                ( )_) |                                                |
|   (_)                 \___/'                                                |
|                                                                             |
| General Bots Copyright (c) Pragmatismo.io. All rights reserved.             |
| Licensed under the AGPL-3.0.                                                |
|                                                                             |
| According to our dual licensing model, this program can be used either      |
| under the terms of the GNU Affero General Public License, version 3,        |
| or under a proprietary license.                                             |
|                                                                             |
| The texts of the GNU Affero General Public License with an additional       |
| permission and of our proprietary license can be found at and               |
| in the LICENSE file you have received along with this program.              |
|                                                                             |
| This program is distributed in the hope that it will be useful,             |
| but WITHOUT ANY WARRANTY, without even the implied warranty of              |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                |
| GNU Affero General Public License for more details.                         |
|                                                                             |
| "General Bots" is a registered trademark of Pragmatismo.io.                 |
| The licensing of the program under the AGPLv3 does not imply a              |
| trademark license. Therefore any rights, title and interest in              |
| our trademarks remain entirely with us.                                     |
|                                                                             |
\*****************************************************************************/
/**
 * @fileoverview General Bots server core.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageDialog = void 0;
const botbuilder_dialogs_1 = require("botbuilder-dialogs");
const botlib_1 = require("botlib");
const strings_1 = require("../strings");
const SecService_1 = require("../../security.gbapp/services/SecService");
const GBConversationalService_1 = require("../services/GBConversationalService");
const pragmatismo_io_framework_1 = require("pragmatismo-io-framework");
/**
 * Dialog for the bot explains about itself.
 */
class LanguageDialog extends botlib_1.IGBDialog {
    /**
     * Setup dialogs flows and define services call.
     *
     * @param bot The bot adapter.
     * @param min The minimal bot instance data.
     */
    static setup(bot, min) {
        min.dialogs.add(new botbuilder_dialogs_1.WaterfallDialog('/language', [
            async (step) => {
                const locale = step.context.activity.locale;
                return await min.conversationalService.prompt(min, step, strings_1.Messages[locale].which_language);
            },
            async (step) => {
                const locale = step.context.activity.locale;
                const user = await min.userProfile.get(step.context, {});
                const list = [
                    { name: 'english', code: 'en' },
                    { name: 'inglês', code: 'en' },
                    { name: 'portuguese', code: 'pt' },
                    { name: 'português', code: 'pt' },
                    { name: 'français', code: 'fr' },
                    { name: 'francês', code: 'fr' },
                    { name: 'french', code: 'fr' },
                    { name: 'português', code: 'pt' },
                    { name: 'spanish', code: 'es' },
                    { name: 'espanõl', code: 'es' },
                    { name: 'espanhol', code: 'es' },
                    { name: 'german', code: 'de' },
                    { name: 'deutsch', code: 'de' },
                    { name: 'alemão', code: 'de' }
                ];
                let translatorLocale = null;
                const text = step.context.activity['originalText'];
                await pragmatismo_io_framework_1.CollectionUtil.asyncForEach(list, async (item) => {
                    if (GBConversationalService_1.GBConversationalService.kmpSearch(text.toLowerCase(), item.name.toLowerCase()) != -1 ||
                        GBConversationalService_1.GBConversationalService.kmpSearch(text.toLowerCase(), item.code.toLowerCase()) != -1) {
                        translatorLocale = item.code;
                    }
                });
                let sec = new SecService_1.SecService();
                user.systemUser = await sec.updateUserLocale(user.systemUser.userId, translatorLocale);
                await min.userProfile.set(step.context, user);
                await min.conversationalService.sendText(min, step, strings_1.Messages[locale].language_chosen);
                await step.replaceDialog('/ask', { firstTime: true });
            }
        ]));
    }
}
exports.LanguageDialog = LanguageDialog;
//# sourceMappingURL=LanguageDialog.js.map