/*****************************************************************************\
|                                               ( )_  _                       |
|    _ _    _ __   _ _    __    ___ ___     _ _ | ,_)(_)  ___   ___     _     |
|   ( '_`\ ( '__)/'_` ) /'_ `\/' _ ` _ `\ /'_` )| |  | |/',__)/' v `\ /'_`\   |
|   | (_) )| |  ( (_| |( (_) || ( ) ( ) |( (_| || |_ | |\__, \| (˅) |( (_) )  |
|   | ,__/'(_)  `\__,_)`\__  |(_) (_) (_)`\__,_)`\__)(_)(____/(_) (_)`\___/'  |
|   | |                ( )_) |                                                |
|   (_)                 \___/'                                                |
|                                                                             |
| General Bots Copyright (c) Pragmatismo.io. All rights reserved.             |
| Licensed under the AGPL-3.0.                                                |
|                                                                             |
| According to our dual licensing model, this program can be used either      |
| under the terms of the GNU Affero General Public License, version 3,        |
| or under a proprietary license.                                             |
|                                                                             |
| The texts of the GNU Affero General Public License with an additional       |
| permission and of our proprietary license can be found at and               |
| in the LICENSE file you have received along with this program.              |
|                                                                             |
| This program is distributed in the hope that it will be useful,             |
| but WITHOUT ANY WARRANTY, without even the implied warranty of              |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                |
| GNU Affero General Public License for more details.                         |
|                                                                             |
| "General Bots" is a registered trademark of Pragmatismo.io.                 |
| The licensing of the program under the AGPLv3 does not imply a              |
| trademark license. Therefore any rights, title and interest in              |
| our trademarks remain entirely with us.                                     |
|                                                                             |
\*****************************************************************************/
/**
 * @fileoverview General Bots server core.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.WelcomeDialog = void 0;
const botbuilder_dialogs_1 = require("botbuilder-dialogs");
const botlib_1 = require("botlib");
const app_1 = require("../../../src/app");
const strings_1 = require("../strings");
/**
 *  Dialog for Welcoming people.
 */
class WelcomeDialog extends botlib_1.IGBDialog {
    /**
     * Setup dialogs flows and define services call.
     *
     * @param bot The bot adapter.
     * @param min The minimal bot instance data.
     */
    static setup(bot, min) {
        min.dialogs.add(new botbuilder_dialogs_1.WaterfallDialog('/', [
            async (step) => {
                if (app_1.GBServer.globals.entryPointDialog !== null &&
                    min.instance.botId === process.env.BOT_ID &&
                    step.context.activity.channelId === 'webchat') {
                    return step.replaceDialog(app_1.GBServer.globals.entryPointDialog);
                }
                const user = await min.userProfile.get(step.context, {});
                const locale = step.context.activity.locale;
                if (!user.once && step.context.activity.channelId === 'webchat'
                    && min.core.getParam(min.instance, 'HelloGoodX', true) === "true") {
                    user.once = true;
                    await min.userProfile.set(step.context, user);
                    const a = new Date();
                    const date = a.getHours();
                    const msg = date < 12
                        ? strings_1.Messages[locale].good_morning
                        : date < 18
                            ? strings_1.Messages[locale].good_evening
                            : strings_1.Messages[locale].good_night;
                    await min.conversationalService.sendText(min, step, strings_1.Messages[locale].hi(msg));
                    await step.replaceDialog('/ask', { firstTime: true });
                    if (step.context.activity !== undefined &&
                        step.context.activity.type === 'message' &&
                        step.context.activity.text !== '') {
                        await step.replaceDialog('/answer', { query: step.context.activity.text });
                    }
                }
                return await step.next();
            }
        ]));
    }
}
exports.WelcomeDialog = WelcomeDialog;
//# sourceMappingURL=WelcomeDialog.js.map