/*****************************************************************************\
|                                               ( )_  _                       |
|    _ _    _ __   _ _    __    ___ ___     _ _ | ,_)(_)  ___   ___     _     |
|   ( '_`\ ( '__)/'_` ) /'_ `\/' _ ` _ `\ /'_` )| |  | |/',__)/' v `\ /'_`\   |
|   | (_) )| |  ( (_| |( (_) || ( ) ( ) |( (_| || |_ | |\__, \| (˅) |( (_) )  |
|   | ,__/'(_)  `\__,_)`\__  |(_) (_) (_)`\__,_)`\__)(_)(____/(_) (_)`\___/'  |
|   | |                ( )_) |                                                |
|   (_)                 \___/'                                                |
|                                                                             |
| General Bots Copyright (c) Pragmatismo.io. All rights reserved.             |
| Licensed under the AGPL-3.0.                                                |
|                                                                             |
| According to our dual licensing model, this program can be used either      |
| under the terms of the GNU Affero General Public License, version 3,        |
| or under a proprietary license.                                             |
|                                                                             |
| The texts of the GNU Affero General Public License with an additional       |
| permission and of our proprietary license can be found at and               |
| in the LICENSE file you have received along with this program.              |
|                                                                             |
| This program is distributed in the hope that it will be useful,             |
| but WITHOUT ANY WARRANTY, without even the implied warranty of              |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                |
| GNU Affero General Public License for more details.                         |
|                                                                             |
| "General Bots" is a registered trademark of Pragmatismo.io.                 |
| The licensing of the program under the AGPLv3 does not imply a              |
| trademark license. Therefore any rights, title and interest in              |
| our trademarks remain entirely with us.                                     |
|                                                                             |
\*****************************************************************************/
/**
 * @fileoverview General Bots server core.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.GBCorePackage = void 0;
const botlib_1 = require("botlib");
const BroadcastDialog_1 = require("./dialogs/BroadcastDialog");
const LanguageDialog_1 = require("./dialogs/LanguageDialog");
const SwitchBot_1 = require("./dialogs/SwitchBot");
const WelcomeDialog_1 = require("./dialogs/WelcomeDialog");
const WhoAmIDialog_1 = require("./dialogs/WhoAmIDialog");
const GBModel_1 = require("./models/GBModel");
/**
 * Package for core.gbapp.
 */
class GBCorePackage {
    constructor() {
        this.CurrentEngineName = 'guaribas-1.0.0';
    }
    async loadPackage(core, sequelize) {
        core.sequelize.addModels([GBModel_1.GuaribasInstance, GBModel_1.GuaribasPackage, GBModel_1.GuaribasChannel, GBModel_1.GuaribasException]);
    }
    async getDialogs(min) {
        botlib_1.GBLog.verbose(`getDialogs called.`);
    }
    async unloadPackage(core) {
        botlib_1.GBLog.verbose(`unloadPackage called.`);
    }
    async unloadBot(min) {
        botlib_1.GBLog.verbose(`unloadBot called.`);
    }
    async onNewSession(min, step) {
        botlib_1.GBLog.verbose(`onNewSession called.`);
    }
    async onExchangeData(min, kind, data) {
        botlib_1.GBLog.verbose(`onExchangeData called.`);
    }
    async loadBot(min) {
        WelcomeDialog_1.WelcomeDialog.setup(min.bot, min);
        WhoAmIDialog_1.WhoAmIDialog.setup(min.bot, min);
        SwitchBot_1.SwitchBotDialog.setup(min.bot, min);
        BroadcastDialog_1.BroadcastDialog.setup(min.bot, min);
        LanguageDialog_1.LanguageDialog.setup(min.bot, min);
    }
}
exports.GBCorePackage = GBCorePackage;
//# sourceMappingURL=index.js.map