/*****************************************************************************\
|                                               ( )_  _                       |
|    _ _    _ __   _ _    __    ___ ___     _ _ | ,_)(_)  ___   ___     _     |
|   ( '_`\ ( '__)/'_` ) /'_ `\/' _ ` _ `\ /'_` )| |  | |/',__)/' v `\ /'_`\   |
|   | (_) )| |  ( (_| |( (_) || ( ) ( ) |( (_| || |_ | |\__, \| (˅) |( (_) )  |
|   | ,__/'(_)  `\__,_)`\__  |(_) (_) (_)`\__,_)`\__)(_)(____/(_) (_)`\___/'  |
|   | |                ( )_) |                                                |
|   (_)                 \___/'                                                |
|                                                                             |
| General Bots Copyright (c) Pragmatismo.io. All rights reserved.             |
| Licensed under the AGPL-3.0.                                                |
|                                                                             |
| According to our dual licensing model, this program can be used either      |
| under the terms of the GNU Affero General Public License, version 3,        |
| or under a proprietary license.                                             |
|                                                                             |
| The texts of the GNU Affero General Public License with an additional       |
| permission and of our proprietary license can be found at and               |
| in the LICENSE file you have received along with this program.              |
|                                                                             |
| This program is distributed in the hope that it will be useful,             |
| but WITHOUT ANY WARRANTY, without even the implied warranty of              |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                |
| GNU Affero General Public License for more details.                         |
|                                                                             |
| "General Bots" is a registered trademark of Pragmatismo.io.                 |
| The licensing of the program under the AGPLv3 does not imply a              |
| trademark license. Therefore any rights, title and interest in              |
| our trademarks remain entirely with us.                                     |
|                                                                             |
\*****************************************************************************/
/**
 * @fileoverview General Bots server core.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.GBImporter = void 0;
const fs = require("fs");
const urlJoin = require("url-join");
const app_1 = require("../../../src/app");
const GBModel_1 = require("../models/GBModel");
const GBConfigService_1 = require("./GBConfigService");
/**
 * Handles the importing of packages.
 */
class GBImporter {
    constructor(core) {
        this.core = core;
    }
    async importIfNotExistsBotPackage(botId, packageName, localPath, additionalInstance = null) {
        const settingsJson = JSON.parse(fs.readFileSync(urlJoin(localPath, 'settings.json'), 'utf8'));
        if (botId === undefined) {
            botId = settingsJson.botId;
        }
        let instance;
        if (botId === undefined) {
            botId = GBConfigService_1.GBConfigService.get('BOT_ID');
            instance = await this.core.loadInstanceByBotId(botId);
            if (!instance) {
                instance = {};
                instance.state = 'active';
                instance.adminPass = GBConfigService_1.GBConfigService.get('ADMIN_PASS');
                instance.botId = GBConfigService_1.GBConfigService.get('BOT_ID');
                instance.cloudSubscriptionId = GBConfigService_1.GBConfigService.get('CLOUD_SUBSCRIPTIONID');
                instance.cloudLocation = GBConfigService_1.GBConfigService.get('CLOUD_LOCATION');
                instance.cloudUsername = GBConfigService_1.GBConfigService.get('CLOUD_USERNAME');
                instance.cloudPassword = GBConfigService_1.GBConfigService.get('CLOUD_PASSWORD');
                instance.marketplaceId = GBConfigService_1.GBConfigService.get('MARKETPLACE_ID');
                instance.marketplacePassword = GBConfigService_1.GBConfigService.get('MARKETPLACE_SECRET');
                instance.storageDialect = GBConfigService_1.GBConfigService.get('STORAGE_DIALECT');
                instance.storageServer = GBConfigService_1.GBConfigService.get('STORAGE_SERVER');
                instance.storageName = GBConfigService_1.GBConfigService.get('STORAGE_NAME');
                instance.storageUsername = GBConfigService_1.GBConfigService.get('STORAGE_USERNAME');
                instance.storagePassword = GBConfigService_1.GBConfigService.get('STORAGE_PASSWORD');
            }
        }
        else {
            instance = await this.core.loadInstanceByBotId(botId);
        }
        if (instance != undefined && instance.botId === null) {
            console.log(`Null BotId after load instance with botId: ${botId}.`);
        }
        else {
            instance = additionalInstance;
        }
        return await this.createOrUpdateInstanceInternal(instance, botId, localPath, settingsJson);
    }
    async createBotInstance(botId) {
        const fullSettingsJson = { ...app_1.GBServer.globals.bootInstance };
        fullSettingsJson.botId = botId;
        return await GBModel_1.GuaribasInstance.create(fullSettingsJson);
    }
    async createOrUpdateInstanceInternal(instance, botId, localPath, settingsJson) {
        const packageJson = JSON.parse(fs.readFileSync(urlJoin(localPath, 'package.json'), 'utf8'));
        const servicesJson = JSON.parse(fs.readFileSync(urlJoin(localPath, 'services.json'), 'utf8'));
        const fullSettingsJson = { ...app_1.GBServer.globals.bootInstance, ...packageJson, ...settingsJson, ...servicesJson };
        if (botId !== undefined) {
            fullSettingsJson.botId = botId;
        }
        if (instance !== null) {
            instance = { ...instance, ...fullSettingsJson };
            return await this.core.saveInstance(instance);
        }
        else {
            return await GBModel_1.GuaribasInstance.create(fullSettingsJson);
        }
    }
}
exports.GBImporter = GBImporter;
//# sourceMappingURL=GBImporterService.js.map