/*****************************************************************************\
|                                               ( )_  _                       |
|    _ _    _ __   _ _    __    ___ ___     _ _ | ,_)(_)  ___   ___     _     |
|   ( '_`\ ( '__)/'_` ) /'_ `\/' _ ` _ `\ /'_` )| |  | |/',__)/' v `\ /'_`\   |
|   | (_) )| |  ( (_| |( (_) || ( ) ( ) |( (_| || |_ | |\__, \| (˅) |( (_) )  |
|   | ,__/'(_)  `\__,_)`\__  |(_) (_) (_)`\__,_)`\__)(_)(____/(_) (_)`\___/'  |
|   | |                ( )_) |                                                |
|   (_)                 \___/'                                                |
|                                                                             |
| General Bots Copyright (c) Pragmatismo.io. All rights reserved.             |
| Licensed under the AGPL-3.0.                                                |
|                                                                             |
| According to our dual licensing model, this program can be used either      |
| under the terms of the GNU Affero General Public License, version 3,        |
| or under a proprietary license.                                             |
|                                                                             |
| The texts of the GNU Affero General Public License with an additional       |
| permission and of our proprietary license can be found at and               |
| in the LICENSE file you have received along with this program.              |
|                                                                             |
| This program is distributed in the hope that it will be useful,             |
| but WITHOUT ANY WARRANTY, without even the implied warranty of              |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                |
| GNU Affero General Public License for more details.                         |
|                                                                             |
| "General Bots" is a registered trademark of Pragmatismo.io.                 |
| The licensing of the program under the AGPLv3 does not imply a              |
| trademark license. Therefore any rights, title and interest in              |
| our trademarks remain entirely with us.                                     |
|                                                                             |
\*****************************************************************************/
/**
 * @fileoverview General Bots server core.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.GBSecurityPackage = void 0;
const botlib_1 = require("botlib");
const OAuthDialog_1 = require("./dialogs/OAuthDialog");
const ProfileDialog_1 = require("./dialogs/ProfileDialog");
const models_1 = require("./models");
/**
 * Package for the security module.
 */
class GBSecurityPackage {
    async getDialogs(min) {
        return [
            ProfileDialog_1.ProfileDialog.getNameDialog(min),
            ProfileDialog_1.ProfileDialog.getEmailDialog(min),
            ProfileDialog_1.ProfileDialog.getMobileDialog(min),
            ProfileDialog_1.ProfileDialog.getMobileConfirmDialog(min),
            OAuthDialog_1.OAuthDialog.getOAuthDialog(min)
        ];
    }
    async unloadPackage(core) {
        botlib_1.GBLog.verbose(`unloadPackage called.`);
    }
    async loadBot(min) {
        botlib_1.GBLog.verbose(`loadBot called.`);
    }
    async unloadBot(min) {
        botlib_1.GBLog.verbose(`unloadBot called.`);
    }
    async onNewSession(min, step) {
        botlib_1.GBLog.verbose(`onNewSession called.`);
    }
    async onExchangeData(min, kind, data) {
        botlib_1.GBLog.verbose(`onExchangeData called.`);
    }
    async loadPackage(core, sequelize) {
        core.sequelize.addModels([models_1.GuaribasGroup, models_1.GuaribasUser, models_1.GuaribasUserGroup]);
    }
}
exports.GBSecurityPackage = GBSecurityPackage;
//# sourceMappingURL=index.js.map